#!/bin/bash

#depwalker
# $Header: /magellan-cvs/magellan-src/mage/usr/lib/mage/depwalker.sh,v 1.9 2008/02/26 19:51:40 niro Exp $

# default die function
die()
{
	echo ${COLRED}"$@"${COLDEFAULT}
	exit 1
}

# include all needed files
[ -f /etc/mage.rc.global ] &&  \
	source /etc/mage.rc.global || \
	die "/etc/mage.rc.global missing"

[ -f ${MAGERC} ] && source ${MAGERC} || \
	die "Your ${MAGERC} is missing. Aborting."

[ -f ${MLIBDIR}/mage4.functions.sh ] && \
	source ${MLIBDIR}/mage4.functions.sh || \
	die "mage functions missing"

# for tests only
# source /home/tjoke/alx-cvs/magellan-src/mage/usr/lib/mage/mage.rc.global
# source /etc/mage.rc
# source /home/tjoke/alx-cvs/magellan-src/mage/usr/lib/mage/mage4.functions.sh
# MLIBDIR=/home/tjoke/alx-cvs/magellan-src/mage/usr/lib/mage

# not serious if missing, only needed for colors
[ -f /etc/init.d/functions ] && source /etc/init.d/functions

# disable colors if wanted
if [[ ${NOCOLORS} = true ]]
then
	COLRED=""
	COLGREEN=""
	COLYELLOW=""
	COLBLUE=""
	COLMAGENTA=""
	COLWHITE=""
	COLDEFAULT=""
fi

usage()
{
	echo
	echo "Usage: $(basename $0) [command] [arg] ..."
	echo
	echo "  -h     --help          shows this help"
	echo "  -c     --pcat          categorie of the package"
	echo "  -n     --pname         name of the package"
	echo "  -v     --pver          version number of the package"
	echo "  -b     --pbuild        build number of the package"
	echo "  -m     --method        which calc method should be used:"
	echo "                         install, srcinstall, depend, srcdepend"
	echo "                         upgrade, srcupgrade"
	echo
	echo "method,name, version and build must be given !"
	echo
	exit 1
}

# very basic getops
for i in $*
do
	case $1 in
		--pcat|-c) shift; PCAT="$1" ;;
		--pname|-n) shift; PNAME="$1" ;;
		--pver|-v) shift; PVER="$1" ;;
		--pbuild|-b) shift; PBUILD="$1" ;;
		--method|-m) shift; METHOD="$1" ;;
		--help|-h) usage ;;
	esac
	shift
done

# sanity checks; abort if not given
[ -z "${PCAT}" ] && usage
[ -z "${PNAME}" ] && usage
[ -z "${PVER}" ] && usage
[ -z "${PBUILD}" ] && usage
[ -z "${METHOD}" ] && usage

# check needed global vars
[ -z "${MAGEDIR}" ] && die "\$MAGEDIR not set."
[ -z "${INSTALLDB}" ] && die "\$INSTALLDB not set."
[ -z "${BUILDDIR}" ] && die "\$BUILDDIR not set."

# other needed vars
ALLDEPS=""
MAGEFILE="${MAGEDIR}/${PCAT}/${PNAME}/${PNAME}-${PVER}-${PBUILD}.mage"

# much faster than fgrep
checklist_alldeps()
{
	local i
	local item="$1"

	for i in ${ALLDEPS}
	do
		[[ ${i} = ${item} ]] && return 1
	done

	return 0
}

checklist_processeddeps()
{
	local i
	local item="$1"

	for i in ${PROCESSEDDEPS}
	do
		[[ ${i} = ${item} ]] && return 1
	done

	return 0
}

#####################
##  rumwandern /path/to/mage/file/.mage
rumwandern()
{
	unset DEPEND
	unset SDEPEND
	unset MY_DEPEND

	local DFILE
	local SYM
	local DEPNAME
	local HIGHEST_DEPFILE
	local MY_DEPEND
	local REAL_PGKNAME
	local VIRTUAL_NAME
	local INSTALL_VIRTUAL
	local PNAME
	local PCAT
	local PVER
	local PBUILD

	DFILE="$1"

	source ${DFILE}
	MY_DEPEND="${DEPEND}"

	# for srcinstall & srcdepend only; SDEPEND also needed
	if [[ ${METHOD} = srcinstall ]] || \
		[[ ${METHOD} = srcpretend ]] || \
		[[ ${METHOD} = srcupgrade ]] || \
		[[ ${METHOD} = srcuppretend ]]
	then
		# only if SDEPEND is not zero
		if [ -n "${SDEPEND}" ]
		then
			# crlf is substantly needed !!
			if [ -n "${MY_DEPEND}" ]
			then
				MY_DEPEND="${MY_DEPEND}
					${SDEPEND}"
			else
				MY_DEPEND="${SDEPEND}"
			fi
		fi
	fi

	unset DEPEND
	unset SDEPEND

	if [ -z "${MY_DEPEND}" ]
	then
		return 1
	fi

	while read SYM DEPNAME
	do
		# exclude empty depnames
		[[ -z ${DEPNAME} ]] && continue

		# exclude all already processed deps -without version
		if ! checklist_processeddeps "${DEPNAME%-*}"
		then
			continue
		fi

		# mark depfile as processed to prevent double runs -without version
		# but do not add any virtuals to PROCESSEDDEPS or their resolved
		# pkgnames will be ignored and they are missing on the dependecy-list
		if [[ ${DEPNAME/virtual\//} = ${DEPNAME} ]]
		then
			PROCESSEDDEPS="${PROCESSEDDEPS} ${DEPNAME%-*}"
		fi

		HIGHEST_DEPFILE=$(dep2highest_magefile "${DEPNAME}")

		PCAT="$(magename2pcat ${HIGHEST_DEPFILE})"
		PNAME="$(magename2pname ${HIGHEST_DEPFILE})"
		PVER="$(magename2pver ${HIGHEST_DEPFILE})"
		PBUILD="$(magename2pbuild ${HIGHEST_DEPFILE})"

		## check ob schon in ALLDEPS enthalten dann mach weiter

		# usage of fgrep is extremly slow and consumes a lot of cpu power
		#if [ -z "$(echo ${ALLDEPS} | fgrep "${HIGHEST_DEPFILE}")" ]
		if checklist_alldeps "${HIGHEST_DEPFILE}"
		then
			### check ob DFILE schon installiert ist ###
			if [ ! -d ${MROOT}${INSTALLDB}/${PCAT}/${PNAME}-${PVER}-${PBUILD} ]
			then
				rumwandern ${HIGHEST_DEPFILE}
				ALLDEPS="${ALLDEPS} ${HIGHEST_DEPFILE}"
			fi
		fi
	done << EOF
${MY_DEPEND}
EOF
	return 0
}


### abort if this package is already installed (retval 3)
# maybe later ?
#[ -d ${INSTALLDB}/${PCAT}/${PNAME}-${PVER}-${PBUILD} ] && exit 3

[[ ${METHOD} = pretend ]] || \
	[[ ${METHOD} = srcpretend ]] || \
	[[ ${METHOD} = uppretend ]] || \
	[[ ${METHOD} = srcuppretend ]] && \
	echo -n "Calculating dependencies ... "


if [[ ${METHOD} = upgrade ]] || \
	[[ ${METHOD} = srcupgrade ]] || \
	[[ ${METHOD} = uppretend ]] || \
	[[ ${METHOD} = srcuppretend ]]
then
	INSTDEPS="$(${MLIBDIR}/magequery.sh -i)"
	for dep in ${INSTDEPS}
	do
		PCAT="$(magename2pcat ${dep} installdb)"
		PNAME="$(magename2pname ${dep})"

		# get the highest mage file from mage-db
		MAGEFILE="$(get_highest_magefile ${PCAT} ${PNAME})"

		# now get the pver&pbuild from the new file
		PVER="$(magename2pver ${MAGEFILE})"
		PBUILD="$(magename2pbuild ${MAGEFILE})"

		# do not wander files which are installed
		if [ ! -d ${INSTALLDB}/${PCAT}/${PNAME}-${PVER}-${PBUILD} ]
		then
			# get dependencies the package
			rumwandern ${MAGEFILE}

			# now add the package itself to the dependencies
			# (if not exists already)
			#if [ -z "$(echo ${ALLDEPS} | fgrep "${MAGEFILE}")" ]
			if checklist_alldeps "${MAGEFILE}"
			then
				ALLDEPS="${ALLDEPS} ${MAGEFILE}"
			fi
		fi
	done
else
	# get dependencies the package
	rumwandern ${MAGEFILE}

	# now add the package itself to the dependencies
	ALLDEPS="${ALLDEPS} ${MAGEFILE}"
fi

[[ ${METHOD} = pretend ]] || \
	[[ ${METHOD} = srcpretend ]] || \
	[[ ${METHOD} = uppretend ]] || \
	[[ ${METHOD} = srcuppretend ]] && \
	echo "done"


## show output of pretend
if [[ ${METHOD} = pretend ]] || \
	[[ ${METHOD} = srcpretend ]] || \
	[[ ${METHOD} = uppretend ]] || \
	[[ ${METHOD} = srcuppretend ]]
then
	# this is a little bit faster
	declare -i x=0
	echo -n "Building dependencies list ... "
	for i in ${ALLDEPS}
	do
		(( x++ ))
		k="$x"
		[ ${x} -le 9  ] && k="0${k}"
		#[ ${x} -le 99  ] && k="0${k}"
		PCAT="$(magename2pcat ${i})"
		PNAME="$(magename2pname ${i})"
		PVER="$(magename2pver ${i})"
		PBUILD="$(magename2pbuild ${i})"
		if [ -z "${list}" ]
		then
			list="\t${COLBLUE}[${k}] ${COLGREEN}${PCAT}/${PNAME}-${PVER}-${PBUILD}${COLDEFAULT}"		
		else
			list="${list}
\t${COLBLUE}[${k}] ${COLGREEN}${PCAT}/${PNAME}-${PVER}-${PBUILD}${COLDEFAULT}"
		fi
		unset PCAT PNAME PVER PBUILD
	done
	echo "done"
	echo -e "${list}"
	echo
fi

## return output from src/install deps
[[ ${METHOD} = install ]] || \
	[[ ${METHOD} = srcinstall ]] || \
	[[ ${METHOD} = upgrade ]] || \
	[[ ${METHOD} = srcupgrade ]] && \
	echo "${ALLDEPS}"

# delete ${BUILDDIR}/virtuals if exists as not needed anymore
#[ -d ${BUILDDIR}/virtuals ] && rm -rf ${BUILDDIR}/virtuals
